#include "../include/YAXServer.h"
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <vgakeyboard.h>

YAXObject *GetObject(int32_t obj)
{
   YAXObjectList *OL;

   for(OL=ObjectListHead;OL!=NULL;OL=OL->next)
      if(OL->object->num==obj) return OL->object;
   return NULL;
}

void YAXReturnInt(int32_t fd, int32_t num)
{
   char str[10];

   bzero(str, 10);
   sprintf(str, "%d ", num);
   write(fd, str, strlen(str));
}

int32_t YAXGetInt(int32_t fd)
{
   char str[10];
   char ch;
   int32_t len, i;

   bzero(str, 10);
   ch=0;
   len=i=0;
   while(ch!=' ') {
      len=read(fd, &ch, 1);
      if(len==1 && ch>=48 && ch<=57) {
         str[i]=ch;
         i++;
      }
   }
   return atoi(str);
}

int32_t YAXNoOp(int32_t fd)	/* This is soooooo useful... really! */
{
   YAXReturnInt(fd, 0);
}

int32_t YAXSetCursor(int32_t fd)
{
   YAXObject *O;
   int32_t obj, m;

   obj=YAXGetInt(fd);
   m=YAXGetInt(fd);
   O=GetObject(obj);
   if(O==NULL) return;
   O->mouse=m;
   if(MouseOldObj==O->num) CurrentMouse=m;
   YAXReturnInt(fd, 0);
}

int32_t YAXGetCursor(int32_t fd)
{
   YAXObject *O;
   int32_t obj;

   obj=YAXGetInt(fd);
   O=GetObject(obj);
   if(O==NULL) return;
   YAXReturnInt(fd, O->mouse);
}

int32_t YAXScreenWidth(int32_t fd)
{
   YAXReturnInt(fd, YAX_Width);
}

int32_t YAXScreenHeight(int32_t fd)
{
   YAXReturnInt(fd, YAX_Height);
}

int32_t YAXBeep(int32_t fd)	/* Ignore the volume for now */
{
   int32_t vol;

   vol=YAXGetInt(fd);
   printf("\a");
   YAXReturnInt(fd, 0);
}

int32_t PlaceOnTop(YAXObject *O)
{
   YAXOverlapList *LL, *LP, *LT;
   YAXObjectList *OL;

   (int)LL=malloc(sizeof(YAXOverlapList));
   LL->object=O;
   LL->x1=O->realx;
   LL->y1=O->realy;
   LL->x2=LL->x1+O->width;
   LL->y2=LL->y1+O->height;
   LL->next=HeadOverlapList;
   HeadOverlapList=LL;
   LP=HeadOverlapList;
   for(LT=HeadOverlapList->next;LT!=NULL;LT=LT->next) {
      if(LT==NULL) break;
      if(LT->object->num==O->num) break;
      LP=LT;
   }
   if(LP!=NULL && LT!=NULL) LP->next=LT->next;
   if(LT!=NULL) free(LT);
   for(OL=ObjectListHead->next;OL!=NULL;OL=OL->next) {
      if(OL->object->parent==O->num)
         PlaceOnTop(OL->object);
   }
}

int32_t EraseObject(YAXObject *O)
{
   YAXOverlapList *LL;
   YAXOverlapList *InvertListHead, *IL;
   char *p;

   gl_setcontext(&YAX_gc);
   gl_setclippingwindow(O->realx, O->realy, O->realx+O->width, O->realy+O->height);
   (int)IL=malloc(sizeof(YAXOverlapList));
   InvertListHead=NULL;
   for(LL=HeadOverlapList;LL!=NULL;LL=LL->next) {
      memcpy(IL, LL, sizeof(YAXOverlapList));
      IL->next=InvertListHead;
      InvertListHead=IL;
      (int)IL=malloc(sizeof(YAXOverlapList));
   }
   free(IL);
   for(LL=InvertListHead;LL!=NULL; LL=LL->next) {
      if(LL->object->num!=O->num) {
         if((LL->x1<=O->realx && LL->x2>=O->realx) || \
          (LL->x1<=O->realx+O->width && LL->x2>=O->realx+O->width) || \
          (LL->y1<=O->realy && LL->y2>=O->realy) || \
          (LL->y1<=O->realy+O->height && LL->y2>=O->realy+O->height) \
         ) {
            gl_putbox(LL->object->realx, LL->object->realy, LL->object->width, LL->object->height, LL->object->look);
         }
      }
   }
   LL=InvertListHead;
   while(LL!=NULL) {
      InvertListHead=LL->next;
      free(LL);
      LL=InvertListHead;
   }
   (int)p=malloc(O->width*O->height);
   gl_setclippingwindow(0, 0, YAX_Width, YAX_Height);
   gl_getbox(O->realx, O->realy, O->width, O->height, p);
   gl_setcontextvga(YAX_Mode);
   hideMouse();
   gl_putbox(O->realx, O->realy, O->width, O->height, p);
   refreshMouse();
   showMouse();
   free(p);
}

int32_t AdjustObjectCoord(YAXObject *O, YAXObject *P, int32_t NewX, int32_t NewY)
{
   YAXObjectList *OL;
   YAXOverlapList *LL;

   if(NewX>=0 && NewX+5<=YAX_Width) O->x=NewX;
   if(NewY>=0 && NewY+5<=YAX_Height) O->y=NewY;
   O->realx=NewX+P->realx;
   O->realy=NewY+P->realy;
   for(LL=HeadOverlapList;LL!=NULL;LL=LL->next)
      if(LL->object->num==O->num) break;
   if(LL!=NULL) {
      LL->x1=O->realx;
      LL->y1=O->realy;
      LL->x2=LL->x1+O->width;
      LL->y2=LL->y1+O->height;
   }
   if(O->look!=NULL) {
      hideMouse();
      gl_putbox(O->realx, O->realy, O->width, O->height, O->look);
      refreshMouse();
      showMouse();
   }
   if(O->isAParent) {
      for(OL=ObjectListHead->next;OL!=NULL;OL=OL->next) {
         if(OL->object->parent==O->num)
            AdjustObjectCoord(OL->object, O, OL->object->x, OL->object->y);
      }
   }
}

int32_t YAXCloseDisplay(int32_t fd)
{
   YAXObjectList *OL, *N;

   printf("Closing %d...\n", fd);
   YAXReturnInt(fd, 0);
   for(OL=ObjectListHead;OL!=NULL;OL=N) {
      if(OL==NULL) break;
      N=OL->next;
      if(OL->object->fd==fd) DoDestroy(OL->object);
   }
   close(fd);
}

void KeyboardEventHandler(int32_t scancode, int32_t press)
{
   YAXObject *O;

   O=GetObject(ActiveObject);
   if(press==KEY_EVENTPRESS && !(O->event_mask&YAXKeyPressEvent))
      YAXThrowEvent(ActiveObject, YAXKeyPressEvent, scancode, 0);
   if(press==KEY_EVENTRELEASE && !(O->event_mask&YAXKeyReleaseEvent))
      YAXThrowEvent(ActiveObject, YAXKeyReleaseEvent, scancode, 0);
}

void YAX_InitFuncStruct()
{
   YAX[YAXNULLOPERATION].func	=YAXNoOp;
   YAX[YAXCREATEOBJECT].func	=YAXCreateObject;
   YAX[YAXEVENTSQUEUED].func	=YAXEventsQueued;
   YAX[YAXSETCURSOR].func	=YAXSetCursor;
   YAX[YAXGETCURSOR].func	=YAXGetCursor;
   YAX[YAXGETEVENT].func	=YAXGetEvent;
   YAX[YAXREMOVEEVENT].func	=YAXRemoveEvent;
   YAX[YAXSCREENWIDTH].func	=YAXScreenWidth;
   YAX[YAXSCREENHEIGHT].func	=YAXScreenHeight;
   YAX[YAXBEEP].func		=YAXBeep;
   YAX[YAXGETOBJECTATTR].func	=YAXGetObjectAttr;
   YAX[YAXSETOBJECTATTR].func	=YAXSetObjectAttr;
   YAX[YAXPUTIMAGE].func	=YAXPutImage;
   YAX[YAXGETIMAGE].func	=YAXGetImage;
   YAX[YAXDESTROYOBJECT].func	=YAXDestroyObject;
   YAX[YAXSETDEFINABLECURSOR].func=YAXSetDefinableCursor;
   YAX[YAXFILL].func		=YAXFill;
   YAX[YAXSETACTIVEOBJECT].func	=YAXSetActiveObject;
   YAX[YAXGETACTIVEOBJECT].func	=YAXGetActiveObject;
   YAX[YAXSENDMESSAGE].func	=YAXSendMessage;
   YAX[YAXMESSAGESQUEUED].func	=YAXMessagesQueued;
   YAX[YAXGETMESSAGE].func	=YAXGetMessage;
   YAX[YAXREMOVEMESSAGE].func	=YAXRemoveMessage;
   YAX[YAXSETCOLOURCELL].func	=YAXSetColourCell;
   YAX[YAXGETCOLOURCELL].func	=YAXGetColourCell;
   YAX[YAXDRAWLINE].func	=YAXDrawLine;
   YAX[YAXGETPIXEL].func	=YAXGetPixel;
   YAX[YAXSETPIXEL].func	=YAXSetPixel;
   YAX[YAXMAKEAPPARENT].func	=YAXMakeApparent;
   YAX[YAXCLOSEDISPLAY].func	=YAXCloseDisplay;
}

