#include <YAXlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>

int32_t YAX_fd;
int32_t YAXEventual;

/***** YAX Initialization/Shutdown *****/
int32_t YAXOpenDisplay(char *display)
{
   struct sockaddr_un yock;

   YAX_fd=socket(AF_UNIX, SOCK_STREAM, 0);
   yock.sun_family=AF_UNIX;
   strcpy(yock.sun_path, "/tmp/YAX.sock");
   YAXEventual=0;
   if(connect(YAX_fd, (struct sockaddr *) &yock, sizeof(yock))==-1) return -1;
   return 0;
}

int32_t YAXCloseDisplay()
{
   int32_t res;

   res=YAXSimpleRequest(1, YAXCLOSEDISPLAY);
   close(YAX_fd);
   return res;
}

int32_t YAXNullOp()
{
   return YAXSimpleRequest(1, YAXNULLOPERATION);
}

int32_t YAXWaitEvent(struct timeval *tv)
{
   fd_set st;
   char che;

   if(YAXEventsQueued()) return 0;
   while(1) {
      FD_ZERO(&st);
      FD_SET(YAX_fd, &st);
      select(100, &st, NULL, NULL, tv);
      read(YAX_fd, &che, 1);
      if(che=='$') return 0;
   }
}

/***** Basic YAX call *****/
int32_t YAXSimpleRequest(int32_t args, int32_t req, ...)
{
   int32_t *idx;

   idx=&req;
   for(args;args>0;args--) {
      YAXSendInt(*idx);
      idx++;
   }
   return YAXGetInt();
}

void YAXSendInt(int32_t num)
{
   char str[10];

   bzero(str, 10);
   sprintf(str, "%d ", num);
   write(YAX_fd, str, strlen(str));
}

int32_t YAXGetInt()
{
   char str[10];
   char ch;
   int32_t len, i;

   bzero(str, 10);
   ch=0;
   len=i=0;
   while(ch!=' ') {
      len=read(YAX_fd, &ch, 1);
      if(len==1 && ch>=48 && ch<=57) {
         str[i]=ch;
         i++;
      }
   }
   return atoi(str);
}

/***** YAX Cursor Interface calls *****/
int32_t YAXSetCursor(int32_t obj, int32_t shape)
{
   return YAXSimpleRequest(3, YAXSETCURSOR, obj, shape);
}

int32_t YAXGetCursor(int32_t obj)
{
   return YAXSimpleRequest(2, YAXGETCURSOR, obj);
}

int32_t YAXSetDefinableCursor(int32_t obj, char *img)
{
   int32_t res;

   res=YAXSimpleRequest(2, YAXSETDEFINABLECURSOR, obj);
   if(res!=0) return res;
   write(YAX_fd, img, 32*32);
   return 0;
}

/***** YAX Object Management calls *****/
int32_t YAXCreateObject(int32_t parent)
{
   return YAXSimpleRequest(2, YAXCREATEOBJECT, parent);
}

int32_t YAXDestroyObject(int32_t obj)
{
   return YAXSimpleRequest(2, YAXDESTROYOBJECT, obj);
}

int32_t YAXSetActiveObject(int32_t obj)
{
   return YAXSimpleRequest(2, YAXSETACTIVEOBJECT, obj);
}

int32_t YAXGetActiveObject()
{
   return YAXSimpleRequest(1, YAXGETACTIVEOBJECT);
}

int32_t YAXGetObjectAttr(int32_t obj, YAXObjectAttr *attr)
{
   int32_t res;

   res=YAXSimpleRequest(2, YAXGETOBJECTATTR, obj);
   if(res!=0) return res;
   read(YAX_fd, attr, sizeof(YAXObjectAttr));
   return 0;
}

int32_t YAXSetObjectAttr(int32_t obj, YAXObjectAttr *attr)
{
   int32_t res;

   res=YAXSimpleRequest(2, YAXSETOBJECTATTR, obj);
   if(res!=0) return res;
   write(YAX_fd, attr, sizeof(YAXObjectAttr));
   YAXGetInt();
   return 0;
}

/***** YAX Graphics Interface calls *****/
int32_t YAXFill(int32_t obj, int32_t x1, int32_t y1, int32_t x2, int32_t y2, int32_t c)
{
   return YAXSimpleRequest(7, YAXFILL, obj, x1, y1, x2, y2, c);
}

int32_t YAXScreenWidth()
{
   return YAXSimpleRequest(1, YAXSCREENWIDTH);
}

int32_t YAXScreenHeight()
{
   return YAXSimpleRequest(1, YAXSCREENHEIGHT);
}

int32_t YAXPutImage(int32_t obj, int32_t x, int32_t y, int32_t map_width, int32_t map_height, char *map)
{
   int32_t res;

   res=YAXSimpleRequest(6, YAXPUTIMAGE, obj, x, y, map_width, map_height);
   if(res!=0) return res;
   write(YAX_fd, map, map_width*map_height);
   return 0;
}

int32_t YAXGetImage(int32_t obj, int32_t x, int32_t y, int32_t map_width, int32_t map_height, char *map)
{
   int32_t res;

   res=YAXSimpleRequest(6, YAXGETIMAGE, obj, x, y, map_width, map_height);
   if(res!=0) return res;
   read(YAX_fd, map, map_width*map_height);
   return 0;
}

int32_t YAXSetColourCell(int32_t cell, int32_t r, int32_t g, int32_t b)
{
   return YAXSimpleRequest(5, YAXSETCOLOURCELL, cell, r, g, b);
}

int32_t YAXGetColourCell(int32_t cell, int32_t *r, int32_t *g, int32_t *b)
{
   *r=YAXSimpleRequest(2, YAXGETCOLOURCELL, cell);
   *g=YAXGetInt();
   *b=YAXGetInt();
   return 0;
}

int32_t YAXSetPalette(int32_t obj, int32_t *pal)
{
   int32_t res;

   res=YAXSimpleRequest(2, YAXSETPALETTE, obj);
   if(res!=0) return res;
   write(YAX_fd, pal, sizeof(int)*256);
   return 0;
}

int32_t YAXGetPalette(int32_t obj, int32_t *pal)
{
   int32_t res;

   res=YAXSimpleRequest(2, YAXGETPALETTE, obj);
   if(res!=0) return res;
   read(YAX_fd, pal, sizeof(int)*256);
   return 0;
}

int32_t YAXSetPixel(int32_t obj, int32_t x, int32_t y, int32_t c)
{
   return YAXSimpleRequest(5, YAXSETPIXEL, obj, x, y, c);
}

int32_t YAXGetPixel(int32_t obj, int32_t x, int32_t y)
{
  return YAXSimpleRequest(4, YAXGETPIXEL, obj, x, y);
}

int32_t YAXDrawLine(int32_t obj, int32_t x1, int32_t y1, int32_t x2, int32_t y2, int32_t c)
{
   return YAXSimpleRequest(7, YAXDRAWLINE, obj, x1, y1, x2, y2, c);
}

/***** YAX Event manager interface *****/
int32_t YAXEventsQueued()
{
   return YAXSimpleRequest(1, YAXEVENTSQUEUED);
}

int32_t YAXGetEvent(int32_t ev, YAXEvent *event)
{
   int32_t res;

   res=YAXSimpleRequest(2, YAXGETEVENT, ev);
   if(res!=0) return res;
   read(YAX_fd, event, sizeof(YAXEvent));
   return 0;
}

int32_t YAXRemoveEvent(int32_t ev)
{
   return YAXSimpleRequest(2, YAXREMOVEEVENT, ev);
}

/***** YAX Message System Calls *****/
int32_t YAXSendMessage(int32_t from, int32_t to, char *mess, int32_t len)
{
   int32_t res;

   res=YAXSimpleRequest(4, YAXSENDMESSAGE, from, to, len);
   if(res!=0) return res;
   write(YAX_fd, mess, len);
   return 0;
}

int32_t YAXMessagesQueued()
{
   return YAXSimpleRequest(1, YAXMESSAGESQUEUED);
}

int32_t YAXGetMessage(int32_t mn, YAXMessage *mess)
{
   int32_t res;

   res=YAXSimpleRequest(2, YAXGETMESSAGE, mn);
   if(res!=0) return res;
   read(YAX_fd, &mess, sizeof(YAXMessage));
   return 0;
}

int32_t YAXRemoveMessage(int32_t mn)
{
   return YAXSimpleRequest(2, YAXREMOVEMESSAGE, mn);
}

/***** Ummm... Other stuff *****/
int32_t YAXBeep(int32_t vol)
{
   return YAXSimpleRequest(2, YAXBEEP, vol);
}

int32_t YAXMakeApparent(int32_t obj)
{
   return YAXSimpleRequest(2, YAXMAKEAPPARENT, obj);
}

